#include <stdio.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_odeiv.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_multimin.h>

#define N_VALUES 4  /* rank of pfaffian */

#define MOVE_T_SUCCESS 1  /* return value of the function move_t */
#define MOVE_T_FAIL    0

#define DEBUG_RK  
/* #define DEBUG_MMIN */

/* ǾΥ르ꥺ */
#define MULTIMIN_FDFMINIMIZER_TYPE gsl_multimin_fdfminimizer_steepest_descent 
/*#define MULTIMIN_FDFMINIMIZER_TYPE gsl_multimin_fminimizer_nmsimplex2 */
/*#define MULTIMIN_FDFMINIMIZER_TYPE gsl_multimin_fdfminimizer_vector_bfgs */
/* #define MULTIMIN_FDFMINIMIZER_TYPE gsl_multimin_fdfminimizer_conjugate_fr  */

/* ʬ򤯥르ꥺ */
#define ODEIV_STEP_TYPE gsl_odeiv_step_rkf45

gsl_matrix *a1, *b1, *c1, *e1;
gsl_matrix *a2, *b2, *c2, *e2;
gsl_matrix *p2, *q2, *p3, *q3, *r3;
gsl_matrix *db11, *db12, *db21, *db22, *dc11, *dc12, *dc21, *dc22, *dq21, *dq22, *dq31, *dq32, *dr31, *dr32;
gsl_matrix *inv_a1;
gsl_matrix *inv_a2;
gsl_matrix *inv_p2;
gsl_matrix *inv_p3;

/* pfaffian */
gsl_matrix *pf1_m;
gsl_matrix *pf2_m;
gsl_matrix *pf11_m;
gsl_matrix *pf12_m;
gsl_matrix *pf21_m;
gsl_matrix *pf22_m;
gsl_matrix *pft_m;

/* gradient */
gsl_vector *grad_v;

/* ߤ */
double *values;
/* ߤΰ */
double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;

/* ¬ */
double g_s11, g_s12, g_s22, g_s1, g_s2, g_s21;

/* move_t ưΰ (MOVE_T_FAIL λ) */
double move_t_points[5];

double my_f(const gsl_vector *v, void *params);
void my_df(const gsl_vector *v, void *param, gsl_vector *df);
void my_fdf(const gsl_vector *x, void *params, double *f, gsl_vector *df);
void gsl_vector_show(gsl_vector *);
void grad(double x11, double x12, double x22, double y1, double y2, double r, double *val);
void move_all(double x11, double x12, double x22, double y1, double y2, double xx11, double xx12, double xx22, double yy1, double yy2, double *val);
int  move_t(double x11, double x12, double x22, double y1, double y2, double xx11, double xx12, double xx22, double yy1, double yy2, double *val);
void move(double y, double yy, double *val, int (*sys_y)(double, const double *, double *, double *));
void move_y1(double y1, double yy1, double *val);
void move_y2(double y2, double yy2, double *val);
void d_rk_printf(char *s);

int sys_t(double t, const double *y, double *val, double *params);
int sys_y1(double y1, const double *y, double *val, double *params);
int sys_y2(double y2, const double *y, double *val, double *params);
int sys_x11(double x11, const double *y, double *val, double *params);
int sys_x12(double x12, const double *y, double *val, double *params);
int sys_x22(double x22, const double *y, double *val, double *params);

void set_abce_1(double x11, double x12, double x22, double y1, double y2, double r);
void set_abce_2(double x11, double x12, double x22, double y1, double y2, double r);
void set_pqr(double x11, double x12, double x22, double y1, double y2, double r);
void set_dbcqr(double x11, double x12, double x22, double y1, double y2, double r);
void pf1(double x11, double x12, double x22, double y1, double y2, double r);
void pf2(double x11, double x12, double x22, double y1, double y2, double r);
void pf11(double x11, double x12, double x22, double y1, double y2, double r);
void pf12(double x11, double x12, double x22, double y1, double y2, double r);
void pf21(double x11, double x12, double x22, double y1, double y2, double r);
void pf22(double x11, double x12, double x22, double y1, double y2, double r);
void invmat(gsl_matrix *m, gsl_matrix *invm);
void gsl_matrix_show(gsl_matrix *mat);

int main(int argc, char *argv[])
{
extern gsl_matrix *a1, *b1, *c1, *e1;
extern gsl_matrix *a2, *b2, *c2, *e2;
extern gsl_matrix *p2, *q2, *p3, *p3, *r3;
extern gsl_matrix *db11, *db12, *db21, *db22, *dc11, *dc12, *dc21, *dc22, *dq21, *dq22, *dq31, *dq32, *dr31, *dr32;
extern gsl_matrix *inv_a1;
extern gsl_matrix *inv_a2;
extern gsl_matrix *inv_p2;
extern gsl_matrix *inv_p3;
extern gsl_matrix *pf1_m;
extern gsl_matrix *pf2_m;
extern gsl_matrix *pf11_m;
extern gsl_matrix *pf12_m;
extern gsl_matrix *pf21_m;
extern gsl_matrix *pf22_m;
extern gsl_matrix *pft_m;

a1 = gsl_matrix_alloc(4, 4);
b1 = gsl_matrix_alloc(4, 4);
c1 = gsl_matrix_alloc(4, 1);
e1 = gsl_matrix_alloc(4, 2);
a2 = gsl_matrix_alloc(4, 4);
b2 = gsl_matrix_alloc(4, 4);
c2 = gsl_matrix_alloc(4, 1);
e2 = gsl_matrix_alloc(4, 2);
p2 = gsl_matrix_alloc(1, 1);
q2 = gsl_matrix_alloc(1, 4);
p3 = gsl_matrix_alloc(2, 2);
q3 = gsl_matrix_alloc(2, 1);
r3 = gsl_matrix_alloc(2, 4);
db11 = gsl_matrix_alloc(4, 4);
db12 = gsl_matrix_alloc(4, 4);
db21 = gsl_matrix_alloc(4, 4);
db22 = gsl_matrix_alloc(4, 4);
dc11 = gsl_matrix_alloc(4, 1);
dc12 = gsl_matrix_alloc(4, 1);
dc21 = gsl_matrix_alloc(4, 1);
dc22 = gsl_matrix_alloc(4, 1);
dq21 = gsl_matrix_alloc(1, 4);
dq22 = gsl_matrix_alloc(1, 4);
dq31 = gsl_matrix_alloc(2, 1);
dq32 = gsl_matrix_alloc(2, 1);
dr31 = gsl_matrix_alloc(2, 4);
dr32 = gsl_matrix_alloc(2, 4);
inv_a1 = gsl_matrix_alloc(4, 4);
inv_a2 = gsl_matrix_alloc(4, 4);
inv_p2 = gsl_matrix_alloc(1, 1);
inv_p3 = gsl_matrix_alloc(2, 2);
pf1_m = gsl_matrix_alloc(4, 4);
pf2_m = gsl_matrix_alloc(4, 4);
pf11_m = gsl_matrix_alloc(4, 4);
pf12_m = gsl_matrix_alloc(4, 4);
pf21_m = gsl_matrix_alloc(4, 4);
pf22_m = gsl_matrix_alloc(4, 4);
pft_m = gsl_matrix_alloc(4, 4);
grad_v = gsl_vector_alloc(5);

extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
extern double g_s11, g_s12, g_s22, g_s1, g_s2, g_s21;
extern double move_t_points[5];

#if 0
	/* move_t  test */
	g_x11 = -3; g_x12 = 0.5; g_x22 = 0.15; g_y1 = 1; g_y2 = 0.15; g_r = 1;
	double val[] = {2.948670364423784504,0.5817985920578351965,0.4754127542095007750,0.6566284657983176130};
	printf("%f %f %f %f %f : val = (%f %f %f %f)\n", g_x11, g_x12, g_x22, g_y1, g_y2, val[0], val[1], val[2], val[3]); 
	move_t(-3, 0.5, 0.15, 1, 0.15, -2, 0.5, 0.15, 1, 0.15, val);
	printf("%f %f %f %f %f : val = (%f %f %f %f)\n", g_x11, g_x12, g_x22, g_y1, g_y2, val[0], val[1], val[2], val[3]); 

	g_x11 = -3; g_x12 = 0.5; g_x22 = 0.15; g_y1 = 1; g_y2 = 0.15; g_r = 1;
	val[0] = 2.948670364423784504;
	val[1] = 0.5817985920578351965;
	val[2] = 0.4754127542095007750;
	val[3] = 0.6566284657983176130;
	printf("%f %f %f %f %f : val = (%f %f %f %f)\n", g_x11, g_x12, g_x22, g_y1, g_y2, val[0], val[1], val[2], val[3]); 
	move_all(-3, 0.5, 0.15, 1, 0.15, -2, 0.5, 0.15, 1, 0.15, val);
	printf("%f %f %f %f %f : val = (%f %f %f %f)\n", g_x11, g_x12, g_x22, g_y1, g_y2, val[0], val[1], val[2], val[3]); 
#endif

	g_x11 = -3; g_x12 = 0.5; g_x22 = 0.15; g_y1 = 1; g_y2 = 0.15; g_r = 1;

	/* exp ʬޤʤ F  
	double val[] = {2.948670364423784504,0.5817985920578351965,0.4754127542095007750,0.6566284657983176130};
	*/

	/* exp ʬޤ᤿ exp() * F  */
	double val[] = {1.602483069690298498,0.3161840010979062376,0.2583676015221306694,0.3568510106161501268};
	values = val;
	double t_x11, t_x12, t_x22, t_y1, t_y2;

	/* S = [0.012,0.313,-0.043,0.501,-0.035]; ¬ */
	/* exp(-(0.012 * x11 + 0.313 * x12 - 0.043 * x22 + 0.501 * y1 + -0.035 * y2)) * FB_integral κǾ */
	g_s11 = 0.012; g_s12 = 0.313; g_s22 = -0.043; g_s1 = 0.501; g_s2 = -0.035; g_s21 = g_s12;

#if 0
/* exp դξ pfaffian ͤΥƥ */
pf11(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
pf12(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
pf22(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
pf1(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
pf2(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
printf("pf11:\n");
gsl_matrix_show(pf11_m);
printf("pf12:\n");
gsl_matrix_show(pf12_m);
printf("pf22:\n");
gsl_matrix_show(pf22_m);
printf("pf1:\n");
gsl_matrix_show(pf1_m);
printf("pf2:\n");
gsl_matrix_show(pf2_m);

/*
Asir Ǥη׻ 

test_pfi_val_gag
pf1:
[ -0.501 1 0 0 ]
[ 0 -0.501 0 1 ]
[ 0.0793651 -0.0238095 -0.34227 -0.15873 ]
[ -0.151036 1.1414 -0.0452512 -0.357445 ]
pf2:
[ 0.035 0 1 0 ]
[ 0.0793651 0.0111905 0.15873 -0.15873 ]
[ 1 0 0.035 -1 ]
[ 0.0365715 -0.105589 0.191108 -0.0367914 ]

test_pfij_val_gag
pf11:
[ -0.012 0 0 1 ]
[ -0.151036 1.1294 -0.0452512 0.143555 ]
[ 0.0365715 -0.105589 0.179108 -0.0717914 ]
[ -0.178152 0.0133113 -0.00219044 1.30812 ]
pf12:
[ -0.233635 -0.0238095 0.15873 -0.15873 ]
[ 0.0365715 -0.418589 0.191108 -0.0717914 ]
[ 0.151036 -0.141403 -0.267749 -0.143555 ]
[ 0.0628745 -0.053608 0.0821503 -0.508382 ]
pf22:
[ 1.043 0 0 -1 ]
[ 0.151036 -0.0984031 0.0452512 -0.143555 ]
[ -0.0365715 0.105589 0.851892 0.0717914 ]
[ 0.178152 -0.0133113 0.00219044 -0.277122 ]
*/
#endif

#if 0
	/* exp դξ move_t Υƥ */
	printf("[%f,%f,%f,%f,%f] : val = (%f %f %f %f)\n", g_x11, g_x12, g_x22, g_y1, g_y2, val[0], val[1], val[2], val[3]); 
	move_all(-3, 0.5, 0.15, 1, 0.15, -3, 0.5, 1.15, 1, 0.15, val);
	g_x22 = 1.15;
	printf("[%f,%f,%f,%f,%f] : val = (%f %f %f %f)\n", g_x11, g_x12, g_x22, g_y1, g_y2, val[0], val[1], val[2], val[3]); 

	/* result from fb1_test in fb1k.rr 
[0,3,[-3,0.5,0.15,1.05,0.15],[1.578698239838109879,0.3259605788043366982,0.2571090666616331849,0.3559803412500595675]]
[1,3,[-3,0.5,0.15,1.1,0.15],[1.555975236510230581,0.3354590002237035657,0.2558750366268017546,0.3553970186273766258]]
[2,3,[-3,0.5,0.15,1.15,0.15],[1.534276944019917693,0.3447005337598251755,0.2546667255078467159,0.3550899894507331928]]
[3,3,[-3,0.5,0.15,1.2,0.15],[1.513565820693715268,0.3537049070055805202,0.2534848893121959826,0.3550484685817667066]]
[4,3,[-3,0.5,0.15,1.25,0.15],[1.493806202631310677,0.3624908458944299980,0.2523302185055065331,0.3552624271421692682]]
[5,3,[-3,0.5,0.15,1.3,0.15],[1.474964209318826175,0.3710761357171153689,0.2512033426956689529,0.3557225483687995943]]
[6,3,[-3,0.5,0.15,1.35,0.15],[1.457007654499408789,0.3794776785201703158,0.2501048350299772792,0.3564201863064590204]]
[7,3,[-3, 0.5,0.15,1.4,0.15],[1.439905961998037817,0.3877115471113423206,0.2490352163228337074,0.3573473271576396694]]

[[ -3 3.05 0.15 2.5 0.15 ],[0.9608608413404185318,0.4814704363008504828,0.4911233 825569663778,0.3596896104526518106]]
	 */
#endif

#if 0
	/* ư fb1k.rr  fb1_test1 ƱưƤ 
	[0,3,[-3,0.5,0.15,1.05,0.15],[1.578698239838109879,0.3259605788043366982,0.2571090666616331849,0.3559803412500595675]]
	[37,3,[-3,0.5,0.15,2.9,0.15],[1.221267921282694797,0.6134944584668179824,0.2316443929610800950,0.4704236713295680955]]
	[38,1,[-3,0.55,0.15,2.9,0.15],[1.208127278266981080,0.6075132471974222644,0.2405382687653997508,0.4655640606164250205]]
	[85,1,[-3,2.9,0.15,2.9,0.15],[0.9693813670339763308,0.5274163750241396186,0.5010153644355929935,0.3921782729133144827]]
	[86,3,[-3,2.9,0.15,2.85,0.15],[0.9674278130692425584,0.5210572917661431575,0.4984109771840218753,0.3874837641983815063]]
	[92,3,[-3,2.9,0.15,2.55,0.15],[0.9615121283088077162,0.4842985957959167292,0.4829435422361759328,0.3617726393181704106]]
	[93,1,[-3,2.95,0.15,2.55,0.15],[0.9612086186085465140,0.4852730001202086387,0.4865911738636830674,0.3623186059146760252]]
	[95,1,[-3,3.05,0.15,2.55,0.15],[0.9610119729231984752,0.4874160529076439767,0.4938326216734649205,0.3635626528578662671]]
	[96,3,[-3,3.05,0.15,2.5,0.15],[0.9608608413404185318,0.4814704363008504828,0.4911233825569663778,0.3596896104526518106]]
	*/

	double points[10][5] = {
		{g_x11, g_x12, g_x22, g_y1, g_y2},
		{-3,0.5,0.15,1.05,0.15},
		{-3,0.5,0.15,2.9,0.15},
		{-3,0.55,0.15,2.9,0.15},
		{-3,2.9,0.15,2.9,0.15},
		{-3,2.9,0.15,2.85,0.15},
		{-3,2.9,0.15,2.55,0.15},
		{-3,2.95,0.15,2.55,0.15},
		{-3,3.05,0.15,2.55,0.15},
		{-3,3.05,0.15,2.5,0.15}
	};


	int i = 0;
	int status;
	printf("[%f,%f,%f,%f,%f] : val = [%f,%f,%f,%f]\n", 
	       points[i][0], points[i][1], points[i][2], points[i][3], points[i][4], val[0], val[1], val[2], val[3]); 
	for (i = 0; i < 9; i++) {
		status = move_t(points[i][0], points[i][1], points[i][2], points[i][3], points[i][4], 
		       points[i+1][0], points[i+1][1], points[i+1][2], points[i+1][3], points[i+1][4], val);
		if (status == MOVE_T_SUCCESS) {
			g_x11 = points[i+1][0]; g_x12 = points[i+1][1]; g_x22 = points[i+1][2]; g_y1 = points[i+1][3]; g_y2 = points[i+1][4];
		} else {
			/* move_t Ԥν */
			printf("move_t failed\n");
			printf("[%f,%f,%f,%f,%f] : val = [%f,%f,%f,%f]\n", 
				move_t_points[0], move_t_points[1], move_t_points[2], move_t_points[3], move_t_points[4], val[0], val[1], val[2], val[3]); 
			break;
		}
		printf("[%f,%f,%f,%f,%f] : val = [%f,%f,%f,%f]\n", 
			points[i+1][0], points[i+1][1], points[i+1][2], points[i+1][3], points[i+1][4], val[0], val[1], val[2], val[3]); 
		pf11(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
		pf12(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
		pf22(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
		pf1(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
		pf2(g_x11, g_x12, g_x22, g_y1, g_y2, g_r);
		/*
		printf("pfaffian:\n");
		gsl_matrix_show(pf11_m); printf("\n");
		gsl_matrix_show(pf12_m); printf("\n");
		gsl_matrix_show(pf22_m); printf("\n");
		gsl_matrix_show(pf1_m); printf("\n");
		gsl_matrix_show(pf2_m); printf("\n");
		*/
	}
#endif


	size_t iter = 0;
	int status;
	const gsl_multimin_fdfminimizer_type *T;
	gsl_multimin_fdfminimizer *s;
	int move_t_status;

	gsl_vector *x;
	gsl_multimin_function_fdf my_func;
	my_func.n = 5;
	my_func.f = &my_f;
	my_func.df = &my_df;
	my_func.fdf = &my_fdf;
	my_func.params = NULL;

	x = gsl_vector_alloc(5);
	gsl_vector_set(x, 0, g_x11);
	gsl_vector_set(x, 1, g_x12);
	gsl_vector_set(x, 2, g_x22);
	gsl_vector_set(x, 3, g_y1);
	gsl_vector_set(x, 4, g_y2);

	T = MULTIMIN_FDFMINIMIZER_TYPE;
	s = gsl_multimin_fdfminimizer_alloc(T, 5);
	gsl_multimin_fdfminimizer_set(s, &my_func, x, 0.01, 1e-4);
	do {
		iter++;
		status = gsl_multimin_fdfminimizer_iterate(s);
		if (status) {
			if (status == GSL_ENOPROG)
				printf("GSL_ENOPROG : gsl_multimin_fdminimizer_iterate\n");
			break;
		}
		status = gsl_multimin_test_gradient(s->gradient, 1e-3);
		if (status == GSL_SUCCESS) printf("Minimum found\n");
		t_x11 = gsl_vector_get(s->x, 0); 
		t_x12 = gsl_vector_get(s->x, 1); 
		t_x22 = gsl_vector_get(s->x, 2); 
		t_y1 = gsl_vector_get(s->x, 3); 
		t_y2 = gsl_vector_get(s->x, 4); 
		printf("%5d %10.5f %10.5f %10.5f %10.5f %10.5f : %10.5f\n", iter, t_x11, t_x12, t_x22, t_y1, t_y2, s->f);
		
		/* ؿ values ι */
		move_t_status = move_t(g_x11, g_x12, g_x22, g_y1, g_y2, t_x11, t_x12, t_x22, t_y1, t_y2, values); 
		if (move_t_status == MOVE_T_SUCCESS) {
			printf("move_t success\n");
			/*  g_* ι */
			g_x11 = t_x11;
			g_x12 = t_x12;
			g_x22 = t_x22;
			g_y1 = t_y1;
			g_y2 = t_y2;
		} else {
			printf("move_t failed\n");
			/*  g_* ι */
			g_x11 = move_t_points[0];
			g_x12 = move_t_points[1];
			g_x22 = move_t_points[2];
			g_y1 = move_t_points[3]; 
			g_y2 = move_t_points[4];
		}
		printf("points = [%f, %f, %f, %f, %f]\n", g_x11, g_x12, g_x22, g_y1, g_y2);
		printf("values = [%f, %f, %f, %f]\n", values[0], values[1], values[2], values[3]);
		printf("grad : "); gsl_vector_show(grad_v);
	} while (status == GSL_CONTINUE && iter < 100);
	gsl_multimin_fdfminimizer_free(s);
	gsl_vector_free(x);

}

/*
֤ȤǤ
[x11,-3],[x12,0.5],[x22,0.15],[y1,1],[y2,0.15],[r,1]
[2.948670364423784504,0.5817985920578351965,0.4754127542095007750,0.6566284657983176130]

[2190] test_fb1_intval3(-3, 0.5, 0.15, 1, 0.15);
[2.948670364423784504,0.5817985920578351965,0.4754127542095007750,0.6566284657983176130]

[2161]  test_fb1_intval3(-2, 0.5, 0.15, 1, 0.15);
[3.829299803048695589,1.003388646378379680,0.5878728001573972805,1.165830164630492881]

[2195] test_fb1_intval3(0.1, 1, 0.1, 1, 0.1);
[9.466729264015772684,4.286998613802449220,1.394076702200820413,5.216053052418173661]

[2197] test_fb1_intval3(-2, 1, 0.1, 1, 0.1);
[3.852525626004445567,1.046519506175361994,0.6435665660589316700,1.208721249400003053]
*/

double my_f(const gsl_vector *v, void *params)
{
extern double *values;
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;

	double o_x11, o_x12, o_x22, o_y1, o_y2;
	double n_x11, n_x12, n_x22, n_y1, n_y2;
	double val[N_VALUES];
	int i;

	/* move_all ǹƤޤΤǡ g_* 򥳥ԡ  */
	for (i = 0; i < N_VALUES; i++)
		val[i] = values[i];
	o_x11 = g_x11; o_x12 = g_x12; o_x22 = g_x22; o_y1 = g_y1; o_y2 = g_y2;
	n_x11 = gsl_vector_get(v, 0);
	n_x12 = gsl_vector_get(v, 1);
	n_x22 = gsl_vector_get(v, 2);
	n_y1 = gsl_vector_get(v, 3);
	n_y2 = gsl_vector_get(v, 4);
printf("call my_f(%f, %f, %f, %f, %f)\n", n_x11, n_x12, n_x22, n_y1, n_y2);
/*
	move_all(g_x11, g_x12, g_x22, g_y1, g_y2, n_x11, n_x12, n_x22, n_y1, n_y2, val);
*/
	move_t(g_x11, g_x12, g_x22, g_y1, g_y2, n_x11, n_x12, n_x22, n_y1, n_y2, val);
	/* g_* 򸵤᤹ͤ */
	g_x11 = o_x11; g_x12 = o_x12; g_x22 = o_x22; g_y1 = o_y1; g_y2 = o_y2;
	return val[0];
}

/* gradient */
void my_df(const gsl_vector *v, void *param, gsl_vector *df)
{
extern double *values;
extern gsl_vector *grad_v;

	double x11, x12, x22, y1, y2; 
	x11 = gsl_vector_get(v, 0);
	x12 = gsl_vector_get(v, 1);
	x22 = gsl_vector_get(v, 2);
	y1 = gsl_vector_get(v, 3);
	y2 = gsl_vector_get(v, 4);
printf("call my_df(%f, %f, %f, %f, %f)\n", x11, x12, x22, y1, y2);

	/*  v Ǥδؿͤ׻ */
	double o_x11, o_x12, o_x22, o_y1, o_y2;
	double n_x11, n_x12, n_x22, n_y1, n_y2;
	double val[N_VALUES];
	int i;
	/* move_all ǹƤޤΤǡ g_* 򥳥ԡ  */
	for (i = 0; i < N_VALUES; i++)
		val[i] = values[i];
	o_x11 = g_x11; o_x12 = g_x12; o_x22 = g_x22; o_y1 = g_y1; o_y2 = g_y2;
	n_x11 = gsl_vector_get(v, 0);
	n_x12 = gsl_vector_get(v, 1);
	n_x22 = gsl_vector_get(v, 2);
	n_y1 = gsl_vector_get(v, 3);
	n_y2 = gsl_vector_get(v, 4);
	move_t(g_x11, g_x12, g_x22, g_y1, g_y2, n_x11, n_x12, n_x22, n_y1, n_y2, val);
	/* g_* 򸵤᤹ͤ */
	g_x11 = o_x11; g_x12 = o_x12; g_x22 = o_x22; g_y1 = o_y1; g_y2 = o_y2;

	/* grad(x11, x12, x22, y1, y2, 1, values); */
	grad(x11, x12, x22, y1, y2, 1, val);
	gsl_vector_set(df, 0, gsl_vector_get(grad_v, 0));
	gsl_vector_set(df, 1, gsl_vector_get(grad_v, 1));
	gsl_vector_set(df, 2, gsl_vector_get(grad_v, 2));
	gsl_vector_set(df, 3, gsl_vector_get(grad_v, 3));
	gsl_vector_set(df, 4, gsl_vector_get(grad_v, 4));
}

void my_fdf(const gsl_vector *x, void *params, double *f, gsl_vector *df)
{
	*f = my_f(x, params);
	my_df(x, params, df);
}

void gsl_vector_show(gsl_vector *v)
{
	int i;
	for (i = 0; i < v->size; i++)
		printf("%f ", gsl_vector_get(v, i)); 
	printf("\n");
}

void grad(double x11, double x12, double x22, double y1, double y2, double r, double *val)
{
extern gsl_vector *grad_v;
	int i;
	double t;

	pf11(x11, x12, x22, y1, y2, r);
	for (i = 0, t = 0; i < N_VALUES; i++)
		t += gsl_matrix_get(pf11_m, 0, i) * val[i];
	gsl_vector_set(grad_v, 0, t);

	pf12(x11, x12, x22, y1, y2, r);
	for (i = 0, t = 0; i < N_VALUES; i++)
		t += gsl_matrix_get(pf12_m, 0, i) * val[i];
	gsl_vector_set(grad_v, 1, t);

	pf22(x11, x12, x22, y1, y2, r);
	for (i = 0, t = 0; i < N_VALUES; i++)
		t += gsl_matrix_get(pf22_m, 0, i) * val[i];
	gsl_vector_set(grad_v, 2, t);

	pf1(x11, x12, x22, y1, y2, r);
	for (i = 0, t = 0; i < N_VALUES; i++)
		t += gsl_matrix_get(pf1_m, 0, i) * val[i];
	gsl_vector_set(grad_v, 3, t);

	pf2(x11, x12, x22, y1, y2, r);
	for (i = 0, t = 0; i < N_VALUES; i++)
		t += gsl_matrix_get(pf2_m, 0, i) * val[i];
	gsl_vector_set(grad_v, 4, t);
}

/* x11,x12,x22,y1,y2,r  xx11,xx12,xx22,yy1,yy2,r ư */
/* g_x11, g_x12, g_x22, g_y1, g_y2 ⹹               */
/* Ȥ Runge-Kutta ư                          */
void move_all(double x11, double x12, double x22, double y1, double y2, double xx11, double xx12, double xx22, double yy1, double yy2, double *val)
{
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
d_rk_printf("move x11\n");
	move(x11, xx11, val, sys_x11);
	g_x11 = xx11;
d_rk_printf("move x12\n");
	move(x12, xx12, val, sys_x12);
	g_x12 = xx12;
d_rk_printf("move x22\n");
	move(x22, xx22, val, sys_x22);
	g_x22 = xx22;
d_rk_printf("move y1\n");
	move(y1, yy1, val, sys_y1);
	g_y1 = yy1;
d_rk_printf("move y2\n");
	move(y2, yy2, val, sys_y2);
	g_y2 = yy2;
}

void d_rk_printf(char *s)
{
#ifdef DEBUG_RK
	printf(s);
#endif
}

/* x11,x12,x22,y1,y2,r  xx11,xx12,xx22,yy1,yy2,r ư */
/* g_x11, g_x12, g_x22, g_y1, g_y2 ⹹               */
/* ٥ȥ (xx11 - x11, xx12 - x12, ... , yy2 - y2) ˱ä Runge-Kutta ư */
int move_t(double x11, double x12, double x22, double y1, double y2, double xx11, double xx12, double xx22, double yy1, double yy2, double *val)
{
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
extern double move_t_points[5];

	/* ٥ȥ */
	double params[5]; 
	params[0] = xx11 - x11;
	params[1] = xx12 - x12;
	params[2] = xx22 - x22;
	params[3] = yy1 - y1;
	params[4] = yy2 - y2;

        const gsl_odeiv_step_type *T = ODEIV_STEP_TYPE;
        gsl_odeiv_step *s = gsl_odeiv_step_alloc(T, 4);
        gsl_odeiv_control *c = gsl_odeiv_control_y_new(1e-6, 0.0);
        gsl_odeiv_evolve *e = gsl_odeiv_evolve_alloc(4);
        gsl_odeiv_system sys = {sys_t, NULL, 4, params};

	double t = 0.0;
	double h = 1e-6;
	double hmin = 1e-6;
#ifdef DEBUG_RK
	printf("Runge-Kutta: [%f,%f,%f,%f,%f] -> [%f,%f,%f,%f,%f]\n", x11, x12, x22, y1, y2, xx11, xx12, xx22, yy1, yy2);
#endif
	while (t < 1.0) {
		int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &t, 1.0, &h, val);
		if (status != GSL_SUCCESS) break;
		if (h < hmin) {
			printf("move_t : h is too small\n");
			move_t_points[0] = g_x11 + t * params[0];
			move_t_points[1] = g_x12 + t * params[1];
			move_t_points[2] = g_x22 + t * params[2];
			move_t_points[3] = g_y1 + t * params[3];
			move_t_points[4] = g_y2 + t * params[4];
			gsl_odeiv_evolve_free(e);
        		gsl_odeiv_control_free(c);
        		gsl_odeiv_step_free(s);
			return MOVE_T_FAIL;
		}
#ifdef DEBUG_RK
		printf("t = %f [%f,%f,%f,%f] : ", t, val[0], val[1], val[2], val[3]);
		printf("[%f,%f,%f,%f,%f]\n", g_x11 + t * params[0], g_x12 + t * params[1], g_x22 + t * params[2], g_y1 + t * params[3], g_y2 + t * params[4]);
#endif
	}
	gsl_odeiv_evolve_free(e);
        gsl_odeiv_control_free(c);
        gsl_odeiv_step_free(s);
	return MOVE_T_SUCCESS;
}

/* params ٥ȥ뤬 */
int sys_t(double t, const double *y, double *val, double *params)
{
extern gsl_matrix *pf11_m, *pf12_m, *pf22_m, *pf1_m, *pf2_m, *pft_m;
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
	int i, j;

	double n_x11, n_x12, n_x22, n_y1, n_y2, n_r;
	n_x11 = g_x11 + t * params[0];
	n_x12 = g_x12 + t * params[1];
	n_x22 = g_x22 + t * params[2];
	n_y1 = g_y1 + t * params[3];
	n_y2 = g_y2 + t * params[4];
	n_r = g_r;
	pf11(n_x11, n_x12, n_x22, n_y1, n_y2, n_r);
	pf12(n_x11, n_x12, n_x22, n_y1, n_y2, n_r);
	pf22(n_x11, n_x12, n_x22, n_y1, n_y2, n_r);
	pf1(n_x11, n_x12, n_x22, n_y1, n_y2, n_r);
	pf2(n_x11, n_x12, n_x22, n_y1, n_y2, n_r);

	gsl_matrix_set_zero(pft_m);
	gsl_matrix_scale(pf11_m, params[0]);
	gsl_matrix_scale(pf12_m, params[1]);
	gsl_matrix_scale(pf22_m, params[2]);
	gsl_matrix_scale(pf1_m,  params[3]);
	gsl_matrix_scale(pf2_m,  params[4]);
	gsl_matrix_add(pft_m, pf11_m);
	gsl_matrix_add(pft_m, pf12_m);
	gsl_matrix_add(pft_m, pf22_m);
	gsl_matrix_add(pft_m, pf1_m);
	gsl_matrix_add(pft_m, pf2_m);

	for (i = 0; i < 4; i++) {
		val[i] = 0.0;
		for (j = 0; j < 4; j++)
			val[i] += gsl_matrix_get(pft_m, i, j) * y[j];
	}	
	return GSL_SUCCESS;
}


/* ѿˤĤ y  yy ޤư */
void move(double y, double yy, double *val, int (*sys_y)(double, const double *, double *, double *))
{
        const gsl_odeiv_step_type *T = ODEIV_STEP_TYPE;
        gsl_odeiv_step *s = gsl_odeiv_step_alloc(T, 4);
        gsl_odeiv_control *c = gsl_odeiv_control_y_new(1e-6, 0.0);
        gsl_odeiv_evolve *e = gsl_odeiv_evolve_alloc(4);
        gsl_odeiv_system sys = {sys_y, NULL, 4, NULL};

	double h = 1e-6;
	if (y <= yy) {
		while (y < yy) {
			int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &y, yy, &h, val);
			if (status != GSL_SUCCESS) break;
#ifdef DEBUG_RK
			printf("%f %f %f %f %f\n", y, val[0], val[1], val[2], val[3]);
#endif
		}
	} else {
		h = -h;
		while (y > yy) {
			int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &y, yy, &h, val);
			if (status != GSL_SUCCESS) break;
#ifdef DEBUG_RK
			printf("%f %f %f %f %f\n", y, val[0], val[1], val[2], val[3]);
#endif
		}

	}
	gsl_odeiv_evolve_free(e);
        gsl_odeiv_control_free(c);
        gsl_odeiv_step_free(s);
}

/* pf1, pf2 ȤäƤ (y1,y2) Ǥͤ顢 (yy1,y2) Ǥͤ */
void move_y1(double y1, double yy1, double *val)
{
        const gsl_odeiv_step_type *T = ODEIV_STEP_TYPE;
        gsl_odeiv_step *s = gsl_odeiv_step_alloc(T, 4);
        gsl_odeiv_control *c = gsl_odeiv_control_y_new(1e-6, 0.0);
        gsl_odeiv_evolve *e = gsl_odeiv_evolve_alloc(4);
        gsl_odeiv_system sys = {sys_y1, NULL, 4, NULL};

	double h = 1e-6;
	if (y1 <= yy1) {
		while (y1 < yy1) {
			int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &y1, yy1, &h, val);
			if (status != GSL_SUCCESS) break;
			printf("%f %f %f %f %f\n", y1, val[0], val[1], val[2], val[3]);
		}
	} else {
		h = -h;
		while (y1 > yy1) {
			int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &y1, yy1, &h, val);
			if (status != GSL_SUCCESS) break;
			printf("%f %f %f %f %f\n", y1, val[0], val[1], val[2], val[3]);
		}

	}
	gsl_odeiv_evolve_free(e);
        gsl_odeiv_control_free(c);
        gsl_odeiv_step_free(s);
}

int sys_y1(double y1, const double *y, double *val, double *params)
{
extern gsl_matrix *pf1_m;
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
	int i, j;

	pf1(g_x11, g_x12, g_x22, y1, g_y2, g_r);
	for (i = 0; i < 4; i++) {
		val[i] = 0.0;
		for (j = 0; j < 4; j++)
			val[i] += gsl_matrix_get(pf1_m, i, j) * y[j];
	}	
	return GSL_SUCCESS;
}

/* pf1, pf2 ȤäƤ (y1,y2) Ǥͤ顢 (y1,yy2) Ǥͤ */
void move_y2(double y2, double yy2, double *val)
{
	/* Runge-Kutta-Fehlberg(4,5) */
        const gsl_odeiv_step_type *T = ODEIV_STEP_TYPE;
        gsl_odeiv_step *s = gsl_odeiv_step_alloc(T, 4);
        gsl_odeiv_control *c = gsl_odeiv_control_y_new(1e-6, 0.0);
        gsl_odeiv_evolve *e = gsl_odeiv_evolve_alloc(4);
        gsl_odeiv_system sys = {sys_y2, NULL, 4, NULL};

	double h = 1e-6;
	if (y2 <= yy2) {
		while (y2 < yy2) {
			int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &y2, yy2, &h, val);
			if (status != GSL_SUCCESS) break;
			printf("%f %f %f %f %f\n", y2, val[0], val[1], val[2], val[3]);
		}
	} else {
		h = -h;
		while (y2 > yy2) {
			int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &y2, yy2, &h, val);
			if (status != GSL_SUCCESS) break;
			printf("%f %f %f %f %f\n", y2, val[0], val[1], val[2], val[3]);
		}

	}
	gsl_odeiv_evolve_free(e);
        gsl_odeiv_control_free(c);
        gsl_odeiv_step_free(s);
}

int sys_y2(double y2, const double *y, double *val, double *params)
{
extern gsl_matrix *pf2_m;
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
	int i, j;

	pf2(g_x11, g_x12, g_x22, g_y1, y2, g_r);
	for (i = 0; i < 4; i++) {
		val[i] = 0.0;
		for (j = 0; j < 4; j++)
			val[i] += gsl_matrix_get(pf2_m, i, j) * y[j];
	}	
	return GSL_SUCCESS;
}

int sys_x11(double x11, const double *y, double *val, double *params)
{
extern gsl_matrix *pf11_m;
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
	int i, j;

	pf11(x11, g_x12, g_x22, g_y1, g_y2, g_r);
	for (i = 0; i < 4; i++) {
		val[i] = 0.0;
		for (j = 0; j < 4; j++)
			val[i] += gsl_matrix_get(pf11_m, i, j) * y[j];
	}	
	return GSL_SUCCESS;
}

int sys_x12(double x12, const double *y, double *val, double *params)
{
extern gsl_matrix *pf12_m;
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
	int i, j;

	pf12(g_x11, x12, g_x22, g_y1, g_y2, g_r);
	for (i = 0; i < 4; i++) {
		val[i] = 0.0;
		for (j = 0; j < 4; j++)
			val[i] += gsl_matrix_get(pf12_m, i, j) * y[j];
	}	
	return GSL_SUCCESS;
}

int sys_x22(double x22, const double *y, double *val, double *params)
{
extern gsl_matrix *pf22_m;
extern double g_x11, g_x12, g_x22, g_y1, g_y2, g_r;
	int i, j;

	pf22(g_x11, g_x12, x22, g_y1, g_y2, g_r);
	for (i = 0; i < 4; i++) {
		val[i] = 0.0;
		for (j = 0; j < 4; j++)
			val[i] += gsl_matrix_get(pf22_m, i, j) * y[j];
	}	
	return GSL_SUCCESS;
}

void set_abce_1(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a1, *b1, *c1, *e1;
gsl_matrix_set(a1, 0, 0, 1);
gsl_matrix_set(a1, 1, 1, 1);
gsl_matrix_set(a1, 2, 1, 1*x12);
gsl_matrix_set(a1, 2, 2, (-2)*x11 + 2*x22);
gsl_matrix_set(a1, 3, 3, 1);
gsl_matrix_set(b1, 0, 1, 1);
gsl_matrix_set(b1, 1, 3, 1);
gsl_matrix_set(b1, 2, 0, 1*x12*r*r);
gsl_matrix_set(b1, 2, 1, (-1)*y2);
gsl_matrix_set(b1, 2, 2, 1*y1);
gsl_matrix_set(b1, 2, 3, (-1)*x12);
gsl_matrix_set(e1, 3, 0, 1);
}

void set_abce_2(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a2, *b2, *c2, *e2;
gsl_matrix_set(a2, 0, 0, 1);
gsl_matrix_set(a2, 1, 1, 2*x11 + (-2)*x22);
gsl_matrix_set(a2, 1, 2, 1*x12);
gsl_matrix_set(a2, 2, 2, 1);
gsl_matrix_set(a2, 3, 3, (-2)*x11 + 2*x22);
gsl_matrix_set(b2, 0, 2, 1);
gsl_matrix_set(b2, 1, 1, 1*y2);
gsl_matrix_set(b2, 1, 2, (-1)*y1);
gsl_matrix_set(b2, 1, 3, 1*x12);
gsl_matrix_set(b2, 2, 0, 1*r*r);
gsl_matrix_set(b2, 2, 3, (-1));
gsl_matrix_set(b2, 3, 1, 1*x12*r*r);
gsl_matrix_set(b2, 3, 2, 1);
gsl_matrix_set(b2, 3, 3, (-1)*y2);
gsl_matrix_set(c2, 3, 0, 1*y1);
gsl_matrix_set(e2, 3, 0, (-2)*x12);
}

void set_pqr(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *p2, *q2, *p3, *q3, *r3;
gsl_matrix_set(p2, 0, 0, (-2)*x11 + 2*x22);
gsl_matrix_set(q2, 0, 0, (-1)*x12*r*r);
gsl_matrix_set(q2, 0, 1, 1*y2);
gsl_matrix_set(q2, 0, 2, (-1)*y1);
gsl_matrix_set(q2, 0, 3, 2*x12);
gsl_matrix_set(p3, 0, 0, 2*x11 + (-2)*x22);
gsl_matrix_set(p3, 0, 1, 2*x12);
gsl_matrix_set(p3, 1, 0, 2*x12);
gsl_matrix_set(p3, 1, 1, (-2)*x11 + 2*x22);
gsl_matrix_set(q3, 0, 0, 1*y2);
gsl_matrix_set(q3, 1, 0, (-1)*y1);
gsl_matrix_set(r3, 0, 0, (-1)*y1*r*r);
gsl_matrix_set(r3, 0, 1, (-2)*x11*r*r + 2*r*r*x22 + 1);
gsl_matrix_set(r3, 0, 2, (-1)*x12*r*r);
gsl_matrix_set(r3, 0, 3, 1*y1);
gsl_matrix_set(r3, 1, 1, (-1)*x12*r*r);
gsl_matrix_set(r3, 1, 2, (-1));
gsl_matrix_set(r3, 1, 3, 1*y2);
}

void set_dbcqr(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *db11, *db12, *db21, *db22, *dc11, *dc12, *dc21, *dc22, *dq21, *dq22, *dq31, *dq32, *dr31, *dr32;
gsl_matrix_set(db11, 2, 2, 1);
gsl_matrix_set(db12, 2, 1, (-1));
gsl_matrix_set(db21, 1, 2, (-1));
gsl_matrix_set(db22, 1, 1, 1);
gsl_matrix_set(db22, 3, 3, (-1));
gsl_matrix_set(dc21, 3, 0, 1);
gsl_matrix_set(dq21, 0, 2, (-1));
gsl_matrix_set(dq22, 0, 1, 1);
gsl_matrix_set(dq31, 1, 0, (-1));
gsl_matrix_set(dq32, 0, 0, 1);
gsl_matrix_set(dr31, 0, 0, (-1)*r*r);
gsl_matrix_set(dr31, 0, 3, 1);
gsl_matrix_set(dr32, 1, 3, 1);
}

void pf1(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a1, *inv_a1, *b1, *c1, *e1;
extern gsl_matrix *p2, *inv_p2, *q2;
extern gsl_matrix *p3, *inv_p3, *q3, *r3;
extern gsl_matrix *pf1_m;
extern double g_s1;

set_abce_1(x11, x12, x22, y1, y2, r);
set_pqr(x11, x12, x22, y1, y2, r);
invmat(a1, inv_a1);
invmat(p2, inv_p2);
invmat(p3, inv_p3);

gsl_matrix *t1 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, q2, 0.0, t1);
gsl_matrix *t2 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, c1, t1, 0.0, t2);
gsl_matrix *t3 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, q3, t1, 0.0, t3);
gsl_matrix_sub(t3, r3);
gsl_matrix *t4 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p3, t3, 0.0, t4);
gsl_matrix *t5 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, e1, t4, 0.0, t5);
gsl_matrix_add(t5, b1);
gsl_matrix_sub(t5, t2);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_a1, t5, 0.0, t2);
gsl_matrix_memcpy(pf1_m, t2);

/* exp() ʬб pfaff Ϥʬ -g_s1 * E ä */
gsl_matrix_set_identity(t2);
gsl_matrix_scale(t2, -g_s1);
gsl_matrix_add(pf1_m, t2);

gsl_matrix_free(t1);
gsl_matrix_free(t2);
gsl_matrix_free(t3);
gsl_matrix_free(t4);
gsl_matrix_free(t5);
}

void pf1_no_diag_shift(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a1, *inv_a1, *b1, *c1, *e1;
extern gsl_matrix *p2, *inv_p2, *q2;
extern gsl_matrix *p3, *inv_p3, *q3, *r3;
extern gsl_matrix *pf1_m;
extern double g_s1;

set_abce_1(x11, x12, x22, y1, y2, r);
set_pqr(x11, x12, x22, y1, y2, r);
invmat(a1, inv_a1);
invmat(p2, inv_p2);
invmat(p3, inv_p3);

gsl_matrix *t1 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, q2, 0.0, t1);
gsl_matrix *t2 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, c1, t1, 0.0, t2);
gsl_matrix *t3 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, q3, t1, 0.0, t3);
gsl_matrix_sub(t3, r3);
gsl_matrix *t4 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p3, t3, 0.0, t4);
gsl_matrix *t5 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, e1, t4, 0.0, t5);
gsl_matrix_add(t5, b1);
gsl_matrix_sub(t5, t2);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_a1, t5, 0.0, t2);
gsl_matrix_memcpy(pf1_m, t2);

gsl_matrix_free(t1);
gsl_matrix_free(t2);
gsl_matrix_free(t3);
gsl_matrix_free(t4);
gsl_matrix_free(t5);
}



void pf2(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a2, *inv_a2, *b2, *c2, *e2;
extern gsl_matrix *p2, *inv_p2, *q2;
extern gsl_matrix *p3, *inv_p3, *q3, *r3;
extern gsl_matrix *pf2_m;
extern double g_s2;

set_abce_2(x11, x12, x22, y1, y2, r);
set_pqr(x11, x12, x22, y1, y2, r);
invmat(a2, inv_a2);
invmat(p2, inv_p2);
invmat(p3, inv_p3);

gsl_matrix *t1 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, q2, 0.0, t1);
gsl_matrix *t2 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, c2, t1, 0.0, t2);
gsl_matrix *t3 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, q3, t1, 0.0, t3);
gsl_matrix_sub(t3, r3);
gsl_matrix *t4 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p3, t3, 0.0, t4);
gsl_matrix *t5 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, e2, t4, 0.0, t5);
gsl_matrix_add(t5, b2);
gsl_matrix_sub(t5, t2);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_a2, t5, 0.0, t2);
gsl_matrix_memcpy(pf2_m, t2);

/* exp() ʬб pfaff Ϥʬ -g_s2 * E ä */
gsl_matrix_set_identity(t2);
gsl_matrix_scale(t2, -g_s2);
gsl_matrix_add(pf2_m, t2);

gsl_matrix_free(t1);
gsl_matrix_free(t2);
gsl_matrix_free(t3);
gsl_matrix_free(t4);
gsl_matrix_free(t5);
}

void pf2_no_diag_shift(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a2, *inv_a2, *b2, *c2, *e2;
extern gsl_matrix *p2, *inv_p2, *q2;
extern gsl_matrix *p3, *inv_p3, *q3, *r3;
extern gsl_matrix *pf2_m;
extern double g_s2;
extern double g_s2;

set_abce_2(x11, x12, x22, y1, y2, r);
set_pqr(x11, x12, x22, y1, y2, r);
invmat(a2, inv_a2);
invmat(p2, inv_p2);
invmat(p3, inv_p3);

gsl_matrix *t1 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, q2, 0.0, t1);
gsl_matrix *t2 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, c2, t1, 0.0, t2);
gsl_matrix *t3 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, q3, t1, 0.0, t3);
gsl_matrix_sub(t3, r3);
gsl_matrix *t4 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p3, t3, 0.0, t4);
gsl_matrix *t5 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, e2, t4, 0.0, t5);
gsl_matrix_add(t5, b2);
gsl_matrix_sub(t5, t2);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_a2, t5, 0.0, t2);
gsl_matrix_memcpy(pf2_m, t2);

gsl_matrix_free(t1);
gsl_matrix_free(t2);
gsl_matrix_free(t3);
gsl_matrix_free(t4);
gsl_matrix_free(t5);
}

void pf11(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a1, *inv_a1, *b1, *c1, *e1;
extern gsl_matrix *a2, *inv_a2, *b2, *c2, *e2;
extern gsl_matrix *p2, *inv_p2, *q2;
extern gsl_matrix *p3, *inv_p3, *q3, *r3;
extern gsl_matrix *pf1_m, *pf2_m;
extern gsl_matrix *pf11_m, *pf12_m, *pf21_m, *pf22_m;
extern gsl_matrix *db11, *db12, *db21, *db22, *dc11, *dc12, *dc21, *dc22, *dq21, *dq22, *dq31, *dq32, *dr31, *dr32;
extern double g_s11;

set_abce_1(x11, x12, x22, y1, y2, r);
set_pqr(x11, x12, x22, y1, y2, r);
set_dbcqr(x11, x12, x22, y1, y2, r);
invmat(a1, inv_a1);
invmat(p2, inv_p2);
invmat(p3, inv_p3);

pf1_no_diag_shift(x11, x12, x22, y1, y2, r);

gsl_matrix *t1 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, q2, 0.0, t1);
gsl_matrix *t2 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, dq21, 0.0, t2);
gsl_matrix *t3 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, dq31, t1, 0.0, t3);
gsl_matrix *t4 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, q3, t2, 0.0, t4);
gsl_matrix_add(t3, t4);
gsl_matrix_sub(t3, dr31);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p3, t3, 0.0, t4);
gsl_matrix *t5 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, e1, t4, 0.0, t5);
gsl_matrix_add(t5, db11); 
gsl_matrix *t6 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, dc11, t1, 0.0, t6);
gsl_matrix_sub(t5, t6); 
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, c1, t2, 0.0, t6);
gsl_matrix_sub(t5, t6); 
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_a1, t5, 0.0, t6);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, pf1_m, pf1_m, 0.0, t5);
gsl_matrix_add(t5, t6); 
gsl_matrix_memcpy(pf11_m, t5);

/* exp() ʬб pfaff Ϥʬ -g_s11 * E ä */
gsl_matrix_set_identity(t5);
gsl_matrix_scale(t5, -g_s11);
gsl_matrix_add(pf11_m, t5);

gsl_matrix_free(t1);
gsl_matrix_free(t2);
gsl_matrix_free(t3);
gsl_matrix_free(t4);
gsl_matrix_free(t5);
gsl_matrix_free(t6);
}

void pf12(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a1, *inv_a1, *b1, *c1, *e1;
extern gsl_matrix *a2, *inv_a2, *b2, *c2, *e2;
extern gsl_matrix *p2, *inv_p2, *q2;
extern gsl_matrix *p3, *inv_p3, *q3, *r3;
extern gsl_matrix *pf1_m, *pf2_m;
extern gsl_matrix *pf11_m, *pf12_m, *pf21_m, *pf22_m;
extern gsl_matrix *db11, *db12, *db21, *db22, *dc11, *dc12, *dc21, *dc22, *dq21, *dq22, *dq31, *dq32, *dr31, *dr32;
extern double g_s12;

set_abce_1(x11, x12, x22, y1, y2, r);
set_pqr(x11, x12, x22, y1, y2, r);
set_dbcqr(x11, x12, x22, y1, y2, r);
invmat(a1, inv_a1);
invmat(p2, inv_p2);
invmat(p3, inv_p3);

pf1_no_diag_shift(x11, x12, x22, y1, y2, r);
pf2_no_diag_shift(x11, x12, x22, y1, y2, r);

gsl_matrix *t1 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, q2, 0.0, t1);
gsl_matrix *t2 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, dq22, 0.0, t2);
gsl_matrix *t3 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, dq32, t1, 0.0, t3);
gsl_matrix *t4 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, q3, t2, 0.0, t4);
gsl_matrix_add(t3, t4);
gsl_matrix_sub(t3, dr32);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p3, t3, 0.0, t4);
gsl_matrix *t5 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, e1, t4, 0.0, t5);
gsl_matrix_add(t5, db12); 
gsl_matrix *t6 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, dc12, t1, 0.0, t6);
gsl_matrix_sub(t5, t6); 
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, c1, t2, 0.0, t6);
gsl_matrix_sub(t5, t6); 
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_a1, t5, 0.0, t6);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, pf1_m, pf2_m, 0.0, t5);
gsl_matrix_add(t5, t6); 
gsl_matrix_memcpy(pf12_m, t5);

/* exp() ʬб pfaff Ϥʬ -g_s12 * E ä */
gsl_matrix_set_identity(t5);
gsl_matrix_scale(t5, -g_s12);
gsl_matrix_add(pf12_m, t5);

gsl_matrix_free(t1);
gsl_matrix_free(t2);
gsl_matrix_free(t3);
gsl_matrix_free(t4);
gsl_matrix_free(t5);
gsl_matrix_free(t6);
}

void pf21(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a1, *inv_a1, *b1, *c1, *e1;
extern gsl_matrix *a2, *inv_a2, *b2, *c2, *e2;
extern gsl_matrix *p2, *inv_p2, *q2;
extern gsl_matrix *p3, *inv_p3, *q3, *r3;
extern gsl_matrix *pf1_m, *pf2_m;
extern gsl_matrix *pf11_m, *pf12_m, *pf21_m, *pf22_m;
extern gsl_matrix *db11, *db12, *db21, *db22, *dc11, *dc12, *dc21, *dc22, *dq21, *dq22, *dq31, *dq32, *dr31, *dr32;
extern double g_s21;

set_abce_2(x11, x12, x22, y1, y2, r);
set_pqr(x11, x12, x22, y1, y2, r);
set_dbcqr(x11, x12, x22, y1, y2, r);
invmat(a2, inv_a2);
invmat(p2, inv_p2);
invmat(p3, inv_p3);

pf2_no_diag_shift(x11, x12, x22, y1, y2, r);
pf1_no_diag_shift(x11, x12, x22, y1, y2, r);

gsl_matrix *t1 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, q2, 0.0, t1);
gsl_matrix *t2 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, dq21, 0.0, t2);
gsl_matrix *t3 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, dq31, t1, 0.0, t3);
gsl_matrix *t4 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, q3, t2, 0.0, t4);
gsl_matrix_add(t3, t4);
gsl_matrix_sub(t3, dr31);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p3, t3, 0.0, t4);
gsl_matrix *t5 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, e2, t4, 0.0, t5);
gsl_matrix_add(t5, db21); 
gsl_matrix *t6 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, dc21, t1, 0.0, t6);
gsl_matrix_sub(t5, t6); 
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, c2, t2, 0.0, t6);
gsl_matrix_sub(t5, t6); 
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_a2, t5, 0.0, t6);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, pf2_m, pf1_m, 0.0, t5);
gsl_matrix_add(t5, t6); 
gsl_matrix_memcpy(pf21_m, t5);

/* exp() ʬб pfaff Ϥʬ -g_s21 * E ä */
gsl_matrix_set_identity(t5);
gsl_matrix_scale(t5, -g_s21);
gsl_matrix_add(pf21_m, t5);

gsl_matrix_free(t1);
gsl_matrix_free(t2);
gsl_matrix_free(t3);
gsl_matrix_free(t4);
gsl_matrix_free(t6);
}

void pf22(double x11, double x12, double x22, double y1, double y2, double r)
{
extern gsl_matrix *a1, *inv_a1, *b1, *c1, *e1;
extern gsl_matrix *a2, *inv_a2, *b2, *c2, *e2;
extern gsl_matrix *p2, *inv_p2, *q2;
extern gsl_matrix *p3, *inv_p3, *q3, *r3;
extern gsl_matrix *pf1_m, *pf2_m;
extern gsl_matrix *pf11_m, *pf12_m, *pf21_m, *pf22_m;
extern gsl_matrix *db11, *db12, *db21, *db22, *dc11, *dc12, *dc21, *dc22, *dq21, *dq22, *dq31, *dq32, *dr31, *dr32;
extern double g_s22;

set_abce_2(x11, x12, x22, y1, y2, r);
set_pqr(x11, x12, x22, y1, y2, r);
set_dbcqr(x11, x12, x22, y1, y2, r);
invmat(a2, inv_a2);
invmat(p2, inv_p2);
invmat(p3, inv_p3);

pf2_no_diag_shift(x11, x12, x22, y1, y2, r);

gsl_matrix *t1 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, q2, 0.0, t1);
gsl_matrix *t2 = gsl_matrix_alloc(1,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p2, dq22, 0.0, t2);
gsl_matrix *t3 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, dq32, t1, 0.0, t3);
gsl_matrix *t4 = gsl_matrix_alloc(2,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, q3, t2, 0.0, t4);
gsl_matrix_add(t3, t4);
gsl_matrix_sub(t3, dr32);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_p3, t3, 0.0, t4);
gsl_matrix *t5 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, e2, t4, 0.0, t5);
gsl_matrix_add(t5, db22); 
gsl_matrix *t6 = gsl_matrix_alloc(4,4);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, dc22, t1, 0.0, t6);
gsl_matrix_sub(t5, t6); 
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, c2, t2, 0.0, t6);
gsl_matrix_sub(t5, t6); 
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, inv_a2, t5, 0.0, t6);
gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, pf2_m, pf2_m, 0.0, t5);
gsl_matrix_add(t5, t6); 
gsl_matrix_memcpy(pf22_m, t5);

/* exp() ʬб pfaff Ϥʬ -g_s22 * E ä */
gsl_matrix_set_identity(t5);
gsl_matrix_scale(t5, -g_s22);
gsl_matrix_add(pf22_m, t5);

gsl_matrix_free(t1);
gsl_matrix_free(t2);
gsl_matrix_free(t3);
gsl_matrix_free(t4);
gsl_matrix_free(t5);
gsl_matrix_free(t6);
}

void invmat(gsl_matrix *m, gsl_matrix *invm)
{
	int n = m->size1;
	int s;
	gsl_permutation *p = gsl_permutation_alloc(n);
	gsl_matrix *old_m = gsl_matrix_alloc(m->size1, m->size2);
	gsl_matrix_memcpy(old_m, m);

	gsl_linalg_LU_decomp(m, p, &s);
	gsl_linalg_LU_invert(m, p, invm);

	gsl_permutation_free(p);
	gsl_matrix_memcpy(m, old_m);
}

void test_invmat()
{
	gsl_matrix *invm = gsl_matrix_alloc(3,3);	
	gsl_matrix *c = gsl_matrix_alloc(3,3);	
	gsl_matrix *mm = gsl_matrix_alloc(3,3);	
	gsl_matrix *mmm = gsl_matrix_alloc(3,3);	

	gsl_matrix_set(mm, 0, 0, 1.0);
	gsl_matrix_set(mm, 0, 1, 2.0);
	gsl_matrix_set(mm, 0, 2, 3.0);
	gsl_matrix_set(mm, 1, 0, 4.0);
	gsl_matrix_set(mm, 1, 2, 5.0);
	gsl_matrix_set(mm, 1, 2, 6.0);
	gsl_matrix_set(mm, 2, 0, 7.0);
	gsl_matrix_set(mm, 2, 1, 1.0);
	gsl_matrix_set(mm, 2, 2, 9.0);
	gsl_matrix_memcpy(mmm, mm);
	invmat(mm, invm);
	gsl_matrix_show(invm);
	gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, mmm, invm, 0.0, c);
	gsl_matrix_show(c);

	gsl_matrix_free(invm);
	gsl_matrix_free(c);
	gsl_matrix_free(mm);
	gsl_matrix_free(mmm);
}

void gsl_matrix_show(gsl_matrix *mat)
{
	int i, j, m, n;

	m = mat->size1;
	n = mat->size2;
	for (i = 0; i < m; i++) {
		for (j = 0; j < n; j++) 
			printf("%8f ", gsl_matrix_get(mat, i, j));
		printf("\n");
	}
}

