/* 
example from taka_runge_kutta.rr
Airy eq 
y'' - ty = 0  
---> 
d/dt y  =  y'
d/dt y' = xy
[y(0), y'(0)] = [0.3550280540, -0.2588194038]
*/
#include <stdio.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_odeiv.h>

/*
d/dt y_i = f_i(t, y_1, ..., y_n) ʤؿ f_i Ϳ
*/
int func (double t, const double y[], double f[], void *params)
{
	f[0] = y[1];
	f[1] = t * y[0];
	return GSL_SUCCESS;
}

/*
Jacobian J = (d/dy_j f_i)
*/
int jac (double t, const double y[], double *dfdy, double dfdt[], void *params)
{
	gsl_matrix_view dfdy_mat 
		= gsl_matrix_view_array (dfdy, 2, 2);
	gsl_matrix * m = &dfdy_mat.matrix; 
	gsl_matrix_set (m, 0, 0, 0.0);
	gsl_matrix_set (m, 0, 1, 1.0);
	gsl_matrix_set (m, 1, 0, t*y[0]);
	gsl_matrix_set (m, 1, 1, 0.0);
	dfdt[0] = 0.0;  /* d/dt f_0 */
	dfdt[1] = y[0];  /* d/dt f_1 */
	return GSL_SUCCESS;
}

int main (void)
{
	 /* 르ꥺ T Ѥ,  2 򤯥ƥå״ؿ */
	/* Runge-Kutta-Fehlberg(4,5) */
	const gsl_odeiv_step_type *T = gsl_odeiv_step_rkf45;
	gsl_odeiv_step *s = gsl_odeiv_step_alloc(T, 2);

	 /* 
	  * ƥƥåפˤƲ y_i(t) Фи 1e-6
	  * и 0.0 ϰޤ褦ʥƥåĴˡ
	  */
	gsl_odeiv_control *c = gsl_odeiv_control_y_new(1e-6, 0.0);

	/* 2 Τεؿ */
	gsl_odeiv_evolve *e = gsl_odeiv_evolve_alloc(2);
	/* gsl_odeiv_system sys = {func, jac, 2, NULL}; */
	gsl_odeiv_system sys = {func, NULL, 2, NULL};
	/* t : start, t1 : goal */
	double t = 0.0, t1 = -5.0;
	double h = -1e-6; /* ƥåν */
	double y[2] = {0.3550280540, -0.2588194038}; /* y ν */

	while (t > t1) {
		int status = gsl_odeiv_evolve_apply(e, c, s, &sys, &t, t1, &h, y);
		if (status != GSL_SUCCESS) break;
		printf("%.5e %.5e %.5e\n", t, y[0], y[1]);
	}

	gsl_odeiv_evolve_free(e);
	gsl_odeiv_control_free(c);
	gsl_odeiv_step_free(s);
	return 0;
}
