/*
	Runge-Kutta
	y_{n+1} = y_n + 1/6 k_1 + 1/3 k_2 + 1/3 k_3 + 1/6 k_4
	k_1 = h f(x_n, y_n)
	k_2 = h f(x_n + 1/2 h, y_n + 1/2 k_1)
	k_3 = h f(x_n + 1/2 h, y_n + 1/2 k_2)
	k_4 = h f(x_n + h, y_n + k_3)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define N 2

/* ٥ȥʹؿ f(x, y) (x : scalar, y : vector) */ 
/* val : return value                              */
void f(double x, double *y, int m, double *val, int n)
{
	val[0] = y[1];
	val[1] = -y[0];
}

void show_v(double *v, int n)
{
	int i;

	for (i = 0; i < n; i++)
		printf("%lf ", v[i]);
}

int main()
{
	int i;
	double x0 = 0.0;
	double y0[N] = {1.0, 0.0};
	double h = 0.01;
	double x; 
	double y[N];
	double xn = 2.0;
	double k1[N], k2[N], k3[N], k4[N];
	double temp[N];
	double ty[N];
	
	for (i = 0; i < N; i++)
		y[i] = y0[i];
	for (x = x0; x < xn; x += h) {
		printf("%lf ", x);
		show_v(y, N);
		printf("|%lf %lf\n", cos(x), -sin(x));

		f(x, y, N, temp, N);
		for (i = 0; i < N; i++)
			k1[i] = h * temp[i];

		for (i = 0; i < N; i++)
			ty[i] = y[i] + 0.5 * k1[i];
		f(x + 0.5 * h, ty, N, temp, N);
		for (i = 0; i < N; i++)
			k2[i] = h * temp[i];

		for (i = 0; i < N; i++)
			ty[i] = y[i] + 0.5 * k2[i];
		f(x + 0.5 * h, ty, N, temp, N);
		for (i = 0; i < N; i++)
			k3[i] = h * temp[i];

		for (i = 0; i < N; i++)
			ty[i] = y[i] + k3[i];
		f(x + h, ty, N, temp, N);
		for (i = 0; i < N; i++)
			k4[i] = h * temp[i];
		
		for (i = 0; i < N; i++)
			y[i] = y[i] + 1.0/6.0 * k1[i] + 1.0/3.0 * k2[i] + 1.0/3.0 * k3[i] + 1.0/6.0 * k4[i];
	}	
	printf("%lf ", x);
	show_v(y, N);
	printf("|%lf %lf\n", cos(x), -sin(x));

	return 0;
}
